#include "devicesettingsdialog.h"
#include "ui_devicesettingsdialog.h"

DeviceSettingsDialog::DeviceSettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DeviceSettingsDialog)
{
    ui->setupUi(this);

    _currentBoardTimers[0] = ui->spinTimerBoard_1->value();
    _currentBoardTimers[1] = ui->spinTimerBoard_2->value();
    _currentBoardTimers[2] = ui->spinTimerBoard_3->value();
    _currentBoardTimers[3] = ui->spinTimerBoard_4->value();

    _currentSpeed = ui->speedBox->currentText().toUInt();

    _currentParity = ui->parityBox->currentIndex();

    for(int i = 0; i < 4; i++)
    {
        _currentAdrs[i] = i+1;
    }
}

DeviceSettingsDialog::~DeviceSettingsDialog()
{
    delete ui;
}

void DeviceSettingsDialog::on_buttonApplyChangeTimer_clicked()
{
    _currentBoardTimers[0] = ui->spinTimerBoard_1->value();
    _currentBoardTimers[1] = ui->spinTimerBoard_2->value();
    _currentBoardTimers[2] = ui->spinTimerBoard_3->value();
    _currentBoardTimers[3] = ui->spinTimerBoard_4->value();
}

void DeviceSettingsDialog::on_buttonApplyChangeSpeed_clicked()
{

    _currentSpeed = ui->speedBox->currentText().toUInt();
    emit speedChanged();
}

void DeviceSettingsDialog::on_buttonApplyChangeParity_clicked()
{
    _currentParity = ui->parityBox->currentData().toUInt();
    emit parityChanged();
}

void DeviceSettingsDialog::on_buttonApplyChangeAdr_clicked()
{
    BoardIdHasBeenChanged* _boardIdHasBeenChanged = new BoardIdHasBeenChanged(ui->idComboBox->currentIndex(), ui->adrSpinBox->value());
    QCoreApplication::postEvent(parent(), _boardIdHasBeenChanged);
}

unsigned DeviceSettingsDialog::currentBoardTimer(unsigned short _ID)
{
    return _currentBoardTimers[_ID];
}

unsigned DeviceSettingsDialog::currentSpeed()
{
    return _currentSpeed;
}

unsigned short DeviceSettingsDialog::currentParity()
{
    return _currentParity;
}

void DeviceSettingsDialog::updateSettingsAfterConnection(unsigned tmp_speed, unsigned tmp_parity, unsigned *tmp_adr)
{
    ui->speedBox->setCurrentText(QString::number(_currentSpeed=tmp_speed, 10));
    if(tmp_parity>0)
        tmp_parity--;
    ui->parityBox->setCurrentIndex(_currentParity = tmp_parity);
    for(int i = 0; i < 4; i++)
    {
        _currentAdrs[i] = tmp_adr[i];
    }
    on_idComboBox_currentIndexChanged(ui->idComboBox->currentIndex());
}

void DeviceSettingsDialog::on_idComboBox_currentIndexChanged(int index)
{
    ui->adrSpinBox->setValue(_currentAdrs[index]);
}
