#ifndef M3KTE_H
#define M3KTE_H

#include <QMainWindow>
#include <QModbusDataUnit>
#include <qprogressbar.h>

#include <QtSerialBus/QModbusDataUnit>
#include "writeregistermodel.h"
#include "devicesettingsdialog.h"

#include <QModbusTcpClient>
#include <QModbusRtuSerialMaster>

#include <QTimer>
#include <QMessageBox>
#include <QProgressDialog>
#include <QErrorMessage>

#include <QtSerialBus/qtserialbusglobal.h>

#if QT_CONFIG(modbus_serialport)
#include <QSerialPort>
#endif

QT_BEGIN_NAMESPACE
namespace Ui { class M3KTE; class SettingsDialog;}
QT_END_NAMESPACE

class SettingsDialog;
class WriteRegisterModel;

class M3KTE : public QMainWindow
{
    Q_OBJECT
private:
    void initActions();
    QModbusDataUnit readRequest() const;
    QModbusDataUnit writeRequest() const;
    void changeTable(int board, int tabletype);
    void debug();
    void errorAdrChange();
    bool event(QEvent* event);
    bool pingNetworkDevices();
    void beginScanBoards();
    void displayResultOfScan(QModbusReply *reply, int boardID);
    void stepForScanCurrentSettings(QModbusReply *reply);
private slots:
    void onConnectClicked();

    void onReadButtonClicked();
    void onReadReady();

    void timeForPingIsGone();

    void checkAdrChange();

    void onWriteButtonClicked();
    void onSelectedBoardChanged(int index);
    void onWriteTableChanged(int index);

    void onSpeedUpdate();
    void onParityUpdate();


    void firstBoardScan();
    void secondBoardScan();
    void thirdBoardScan();
    void fourthBoardScan();

    void firstBoardReady();
    void secondBoardReady();
    void thirdBoardReady();
    void fourthBoardReady();
public:
    M3KTE(QWidget *parent = nullptr);
    ~M3KTE();

private:
    bool timerForPingSignal = false;



    Ui::M3KTE *ui;
    //int DeviceOnNetwork[4];
    QProgressBar *m[320];
    QModbusReply *lastRequest = nullptr;
    QModbusClient *modbusDevice = nullptr;
    DeviceSettingsDialog *m_deviceSettingsDialog = nullptr;
    SettingsDialog *m_settingsDialog = nullptr;
    //WriteRegisterModel *writeModel = nullptr;

    struct StatusM3KTE{
        bool Warnings[4];
        bool Accidents[4];
    }statusM3KTE;

    struct BoardModbusRegisters
    {
        int adr;
        int _tmp_adr;
        WriteRegisterModel *ModbusModelCoil;
        WriteRegisterModel *ModbusModelHoldingReg;
        QTimer *boardScanners;
    }Boards[4];
};
#endif // M3KTE_H
